@@ -22,16 +22,13 @@ from tqdm import tqdm
 from . import binning, pid_data
 
 
-def make_hist(
-    df: pd.DataFrame, particle: str, bin_vars: List[str], square_weights: bool = False
-) -> bh.Histogram:
+def make_hist(df: pd.DataFrame, particle: str, bin_vars: List[str]) -> bh.Histogram:
     """Create a histogram of sWeighted events with appropriate binning
 
     Args:
         df: DataFrame from which to histogram events.
         particle: Particle type (K, Pi, etc.).
         bin_vars: Binning variables in the user-convention, e.g., ["P", "ETA"].
-        square_weights: Use square of sWeights instead of sWeights as the weights.
     """
     axis_list = []
     vals_list = []
@@ -44,11 +41,8 @@ def make_hist(
         vals_list.append(vals)
 
     # Create boost-histogram with the desired axes, and fill with sWeight applied
-    hist = bh.Histogram(*axis_list)
-    if square_weights:
-        hist.fill(*vals_list, weight=np.square(df["sWeight"]))  # type: ignore
-    else:
-        hist.fill(*vals_list, weight=df["sWeight"])
+    hist = bh.Histogram(*axis_list, storage=bh.storage.Weight())
+    hist.fill(*vals_list, weight=df["sWeight"])
 
     return hist
 
@@ -57,16 +51,13 @@ def create_eff_histograms(hists: Dict[str, bh.Histogram]) -> Dict[str, bh.Histog
     """Create efficiency histograms for all supplied PID cuts.
 
     Args:
-        df_total: DataFrame with all events.
-        particle: Particle type (K, Pi, etc.).
-        pid_cuts: Simple user-level cut list, e.g., ["DLLK<4"].
-        bin_vars: Variables used for binning.
+        hists: A dictionary with total and passing histograms.
 
     Returns:
         A dictionary with all the efficiency histograms, with the PID cuts as
         keys.
     """
-    zero_bins = np.count_nonzero(hists["total"].view(flow=False) == 0)
+    zero_bins = np.count_nonzero(hists["total"].values(flow=False) == 0)
     if zero_bins:
         log.warning(
             (
@@ -74,22 +65,48 @@ def create_eff_histograms(hists: Dict[str, bh.Histogram]) -> Dict[str, bh.Histog
                 "You might want to change the binning."
             )
         )
-        log.debug(hists["total"].view(flow=False))
+        log.debug(hists["total"].values(flow=False))
 
         # Replace zeros with NaNs which suppresses duplicate Numpy warnings
-        hist_total_nan = hists["total"].view()
-        hist_total_nan[hist_total_nan == 0] = np.nan  # type: ignore
-        hists["total"][...] = hist_total_nan
+        hist_total_nan = hists["total"].values()
+        hist_total_nan[hist_total_nan == 0] = np.nan
+        hists["total"].view().value = hist_total_nan  # type: ignore
 
     for name in list(hists):
-        if name.startswith("passing_") and not name.startswith("passing_sumw2_"):
+        if name.startswith("passing_"):
             eff_name = name.replace("passing_", "eff_", 1)
             hists[eff_name] = hists[name].copy()
-            hists[eff_name][...] = hists[name].view(flow=False) / hists["total"].view(
+            hists[eff_name].view().value = hists[name].values(flow=False) / hists[  # type: ignore # noqa
+                "total"
+            ].values(
                 flow=False
-            )  # type: ignore
+            )
+            hists[eff_name].view().variance = np.square(  # type: ignore
+                binomial_uncertainty(
+                    hists[name].values(flow=False),
+                    hists["total"].values(flow=False),
+                    hists[name].variances(flow=False),  # type: ignore
+                    hists["total"].variances(flow=False),  # type: ignore
+                )
+            )
             log.debug(f"Created '{eff_name}' histogram")
 
+            negative_bins = np.count_nonzero(hists[eff_name].values(flow=False) < 0)
+            if negative_bins:
+                log.warning(
+                    (
+                        f"There are {negative_bins} negative bins in the '{eff_name}' "
+                        "efficiency histogram! You might want to change the binning."
+                    )
+                )
+                log.debug(hists[eff_name].values(flow=False))
+
+    if zero_bins:
+        # Return the zeros that were temporarily replaced by NaNs
+        hists["total"].view().value = np.nan_to_num(  # type:ignore
+            hists["total"].view().value  # type:ignore
+        )
+
     return hists
 
 
@@ -155,7 +172,7 @@ def add_bin_indices(
         df_new.dropna(inplace=True)
         index_names = [f"{axis}_PIDCalibBin" for axis in axes]
         indices = np.ravel_multi_index(
-            df_new[index_names].transpose().to_numpy().astype(int),  # type: ignore
+            df_new[index_names].transpose().to_numpy().astype(int),
             eff_histo.axes.size,
         )
         df_new[f"{prefix}_PIDCalibBin"] = indices
@@ -186,17 +203,16 @@ def add_efficiencies(
     # We separate the dataframe into two parts: one where all the events have
     # PID indices (events inside the PID binning) and those that don't.
     # Efficiency is added only for events inside the PID binning.
-    df_nan = df_new[df_new.isna().any(axis=1)]
+    df_nan = df_new[df_new.isna().any(axis=1)]  # type: ignore
     df_new.dropna(inplace=True)
 
     df_new["PIDCalibEff"] = 1
     df_new["PIDCalibRelErr2"] = 0
 
     for prefix in prefixes:
-        efficiency_table = eff_hists[prefix]["eff"].view().flatten()  # type: ignore
-        error_table = (
-            create_error_histogram(eff_hists[prefix]).view().flatten()  # type: ignore
-        )
+        efficiency_table = eff_hists[prefix]["eff"].values().flatten()
+        error_table = np.sqrt(eff_hists[prefix]["eff"].variances().flatten())  # type: ignore # noqa
+
         # The original PIDCalib assigned bins with no events in the total
         # histogram an efficiency of zero. This should not come up often and the
         # user should be warned about it. In any case it does not seem right -
@@ -220,11 +236,11 @@ def add_efficiencies(
 
     df_new["PIDCalibErr"] = np.sqrt(df_new["PIDCalibRelErr2"])  # type: ignore
     for prefix in prefixes:
-        df_new["PIDCalibErr"] *= df_new[f"{prefix}_PIDCalibEff"]
+        df_new["PIDCalibErr"] *= df_new[f"{prefix}_PIDCalibEff"]  # type: ignore
 
     df_new.drop(columns=["PIDCalibRelErr2"], inplace=True)
 
-    df_new = pd.concat([df_new, df_nan]).sort_index()  # type: ignore
+    df_new = pd.concat([df_new, df_nan]).sort_index()
     log.debug("Particle efficiencies assigned")
 
     num_outside_range = len(df_nan.index)
@@ -356,7 +372,9 @@ def find_similar_strings(
     """
     similar_strings = {}
     for string in list_of_strings:
-        string_ratio = difflib.SequenceMatcher(None, comparison_string, string).ratio()
+        string_ratio = difflib.SequenceMatcher(
+            None, comparison_string.lower(), string.lower()
+        ).ratio()
         if string_ratio > ratio:
             similar_strings[string] = string_ratio
 
@@ -397,7 +415,11 @@ def create_histograms(config):
             config["samples_file"],
             config["max_files"],
         )
-    tree_paths = pid_data.get_tree_paths(config["particle"], config["sample"])
+    tree_paths = pid_data.get_tree_paths(
+        config["particle"],
+        config["sample"],
+        calib_sample["tuple_names"] if "tuple_names" in calib_sample else None,
+    )
     log.debug(f"Trees to be read: {tree_paths}")
 
     # If there are hard-coded cuts, the variables must be included in the
@@ -422,7 +444,7 @@ def create_histograms(config):
     for bin_var in config["bin_vars"]:
         bin_edges = binning.get_binning(config["particle"], bin_var, verbose=True)
         binning_range_cuts.append(
-            f"{bin_var} > {bin_edges[0]} and {bin_var} < {bin_edges[-1]}"
+            f"{bin_var} >= {bin_edges[0]} and {bin_var} < {bin_edges[-1]}"
         )
 
     cut_stats = {
@@ -432,7 +454,9 @@ def create_histograms(config):
     }
     all_hists = {}
     for path in tqdm(calib_sample["files"], leave=False, desc="Processing files"):
-        df = pid_data.root_to_dataframe(path, tree_paths, list(branch_names.values()))
+        df = pid_data.root_to_dataframe(
+            path, tree_paths, list(branch_names.values()), True
+        )
         if df is not None:
             # Rename colums of the dataset from branch names to simple user-level
             # names, e.g., probe_PIDK -> DLLK.
@@ -449,9 +473,6 @@ def create_histograms(config):
 
             hists = {}
             hists["total"] = make_hist(df, config["particle"], config["bin_vars"])
-            hists["total_sumw2"] = make_hist(
-                df, config["particle"], config["bin_vars"], True
-            )
 
             hists_passing = create_passing_histograms(
                 df,
@@ -487,15 +508,11 @@ def create_histograms_from_local_dataframe(config):
 
     hists = {}
     hists["total"] = make_hist(df, particle, bin_vars)
-    hists["total_sumw2"] = make_hist(df, particle, bin_vars, True)
 
     for i, pid_cut in enumerate(pid_cuts):
         log.info(f"Processing '{pid_cuts[i]}' cut")
         df_passing = df.query(pid_cut)
         hists[f"passing_{pid_cut}"] = make_hist(df_passing, particle, bin_vars)
-        hists[f"passing_sumw2_{pid_cut}"] = make_hist(
-            df_passing, particle, bin_vars, True
-        )
         log.debug("Created 'passing' histogram")
 
     return hists
@@ -509,10 +526,11 @@ def apply_all_cuts(
     user_cuts: List[str],
 ) -> Dict[str, Dict[str, int]]:
 
-    log.debug(f"Applying binning range cuts: {binning_range_cuts}'")
-    num_before, num_after = apply_cuts(df, binning_range_cuts)
-    cut_stats["binning range"]["before"] += num_before
-    cut_stats["binning range"]["after"] += num_after
+    if binning_range_cuts:
+        log.debug(f"Applying binning range cuts: {binning_range_cuts}'")
+        num_before, num_after = apply_cuts(df, binning_range_cuts)
+        cut_stats["binning range"]["before"] += num_before
+        cut_stats["binning range"]["after"] += num_after
 
     if hardcoded_cuts:
         log.debug(f"Applying hard-coded cuts: {hardcoded_cuts}'")
@@ -536,9 +554,6 @@ def create_passing_histograms(df, cut_stats, particle, bin_vars, pid_cuts):
         log.debug(f"Processing '{pid_cuts[i]}' cut")
         df_passing = df.query(pid_cut)
         hists[f"passing_{pid_cut}"] = make_hist(df_passing, particle, bin_vars)
-        hists[f"passing_sumw2_{pid_cut}"] = make_hist(
-            df_passing, particle, bin_vars, True
-        )
         log.debug("Created 'passing' histogram")
         if f"'{pid_cut}'" not in cut_stats:
             cut_stats[f"'{pid_cut}'"] = {"before": 0, "after": 0}
